@@ -3,8 +3,8 @@ use strict;
 use warnings;
 package Email::MIME;
 # ABSTRACT: easy MIME message handling
-$Email::MIME::VERSION = '1.928';
-use Email::Simple 2.102; # crlf handling
+$Email::MIME::VERSION = '1.932';
+use Email::Simple 2.104; # header_raw_*
 use parent qw(Email::Simple);
 
 use Carp ();
@@ -72,7 +72,7 @@ use Scalar::Util qw(reftype);
 #pod         Email::MIME->create( parts => [ @parts ] ),
 #pod       ],
 #pod   );
-#pod   
+#pod
 #pod   # standard modifications
 #pod   $email->header_str_set( 'X-PoweredBy' => 'RT v3.0'      );
 #pod   $email->header_str_set( To            => rcpts()        );
@@ -81,7 +81,7 @@ use Scalar::Util qw(reftype);
 #pod
 #pod   # more advanced
 #pod   $_->encoding_set( 'base64' ) for $email->parts;
-#pod   
+#pod
 #pod   # Quick multipart creation
 #pod   my $quicky = Email::MIME->create(
 #pod       header_str => [
@@ -94,9 +94,9 @@ use Scalar::Util qw(reftype);
 #pod           q[These could be binary too],
 #pod       ],
 #pod   );
-#pod   
+#pod
 #pod   print $email->as_string;
-#pod   
+#pod
 #pod =head1 DESCRIPTION
 #pod
 #pod This is an extension of the L<Email::Simple> module, to handle MIME
@@ -169,6 +169,12 @@ sub new {
 #pod
 #pod =cut
 
+my %CT_SETTER = map {; $_ => 1 } qw(
+  content_type charset name format boundary
+  encoding
+  disposition filename
+);
+
 sub create {
   my ($class, %args) = @_;
 
@@ -218,13 +224,8 @@ sub create {
 
   my $email = $class->new($header);
 
-  foreach (qw(
-    content_type charset name format boundary
-    encoding
-    disposition filename
-  )) {
-    my $set = "$_\_set";
-    $email->$set($attrs{$_}) if exists $attrs{$_};
+  for my $key (keys %attrs) {
+    $email->content_type_attribute_set($key => $attrs{$key});
   }
 
   my $body_args = grep { defined $args{$_} } qw(parts body body_str);
@@ -432,6 +433,11 @@ sub invent_filename {
 
 sub default_header_class { 'Email::MIME::Header' }
 
+sub header_str {
+  my $self = shift;
+  $self->header_obj->header_str(@_);
+}
+
 sub header_str_set {
   my $self = shift;
   $self->header_obj->header_str_set(@_);
@@ -512,6 +518,30 @@ sub boundary_set {
   $self->parts_set([ $self->parts ]) if $self->parts > 1;
 }
 
+sub content_type_attribute_set {
+  my ($self, $key, $value) = @_;
+  $key = lc $key;
+
+  if ($CT_SETTER{$key}) {
+    my $method = "$key\_set";
+    return $self->$method($value);
+  }
+
+  my $ct_header = parse_content_type($self->header('Content-Type'));
+  my $attrs = $ct_header->{attributes};
+
+  for my $existing_key (keys %$attrs) {
+    delete $attrs->{$existing_key} if lc $existing_key eq $key;
+  }
+
+  if ($value) {
+    $ct_header->{attributes}->{$key} = $value;
+  } else {
+    delete $ct_header->{attributes}->{$key};
+  }
+  $self->_compose_content_type($ct_header);
+}
+
 #pod =method encoding_set
 #pod
 #pod   $email->encoding_set( 'base64' );
@@ -529,7 +559,7 @@ sub encoding_set {
   my ($self, $enc) = @_;
   $enc ||= '7bit';
   my $body = $self->body;
-  $self->header_set('Content-Transfer-Encoding' => $enc);
+  $self->header_raw_set('Content-Transfer-Encoding' => $enc);
   $self->body_set($body);
 }
 
@@ -553,7 +583,6 @@ sub body_set {
     Carp::croak("provided body reference is not a scalar reference")
       unless reftype($body) eq 'SCALAR';
     $body_ref = $body;
-    $body     = $$body_ref;
   } else {
     $body_ref = \$body;
   }
@@ -565,11 +594,11 @@ sub body_set {
   # Simple subclasses were free to alter the guts of the Email::Simple
   # object. -- rjbs, 2007-07-16
   unless (((caller(1))[3] || '') eq 'Email::Simple::new') {
-    $body = Email::MIME::Encodings::encode($enc, $body)
+    $$body_ref = Email::MIME::Encodings::encode($enc, $$body_ref)
       unless !$enc || $enc =~ $NO_ENCODE_RE;
   }
 
-  $self->{body_raw} = $body;
+  $self->{body_raw} = $$body_ref;
   $self->SUPER::body_set($body_ref);
 }
 
@@ -614,7 +643,7 @@ sub disposition_set {
   $dis_header
     ? ($dis_header =~ s/^([^;]+)/$dis/)
     : ($dis_header = $dis);
-  $self->header_set('Content-Disposition' => $dis_header);
+  $self->header_raw_set('Content-Disposition' => $dis_header);
 }
 
 #pod =method filename_set
@@ -643,7 +672,7 @@ sub filename_set {
     $dis .= qq[; $attr="$val"];
   }
 
-  $self->header_set('Content-Disposition' => $dis);
+  $self->header_raw_set('Content-Disposition' => $dis);
 }
 
 #pod =method parts_set
@@ -679,9 +708,12 @@ sub parts_set {
     delete $ct_header->{attributes}{charset};
   } elsif (@$parts == 1) {  # setup singlepart
     $body .= $parts->[0]->body;
-    @{$ct_header}{qw[type subtype]}
-      = @{ parse_content_type($parts->[0]->header('Content-Type')) }
-      {qw[type subtype]};
+
+    my $from_ct = parse_content_type($parts->[0]->header('Content-Type'));
+    @{$ct_header}{qw[type subtype]} = @{ $from_ct }{qw[type subtype]};
+
+    $ct_header->{attributes}{charset} = $from_ct->{attributes}{charset};
+
     $self->encoding_set($parts->[0]->header('Content-Transfer-Encoding'));
     delete $ct_header->{attributes}->{boundary};
   }
@@ -712,7 +744,7 @@ sub parts_add {
 #pod   $email->walk_parts(sub {
 #pod       my ($part) = @_;
 #pod       return if $part->subparts; # multipart
-#pod       
+#pod
 #pod       if ( $part->content_type =~ m[text/html]i ) {
 #pod           my $body = $part->body;
 #pod           $body =~ s/<link [^>]+>//; # simple filter example
@@ -775,7 +807,7 @@ sub _compose_content_type {
   for my $attr (sort keys %{ $ct_header->{attributes} }) {
     $ct .= qq[; $attr="$ct_header->{attributes}{$attr}"];
   }
-  $self->header_set('Content-Type' => $ct);
+  $self->header_raw_set('Content-Type' => $ct);
   $self->{ct} = $ct_header;
 }
 
@@ -801,11 +833,11 @@ sub _reset_cids {
       return if keys(%cids) == 1;
 
       my $cid = $self->_get_cid;
-      $_->header_set('Content-ID' => "<$cid>") for $self->parts;
+      $_->header_raw_set('Content-ID' => "<$cid>") for $self->parts;
     } else {
       foreach ($self->parts) {
         my $cid = $self->_get_cid;
-        $_->header_set('Content-ID' => "<$cid>")
+        $_->header_raw_set('Content-ID' => "<$cid>")
           unless $_->header('Content-ID');
       }
     }
@@ -824,7 +856,7 @@ Email::MIME - easy MIME message handling
 
 =head1 VERSION
 
-version 1.928
+version 1.932
 
 =head1 SYNOPSIS
 
@@ -880,7 +912,7 @@ by all means keep reading.
         Email::MIME->create( parts => [ @parts ] ),
       ],
   );
-  
+
   # standard modifications
   $email->header_str_set( 'X-PoweredBy' => 'RT v3.0'      );
   $email->header_str_set( To            => rcpts()        );
@@ -889,7 +921,7 @@ by all means keep reading.
 
   # more advanced
   $_->encoding_set( 'base64' ) for $email->parts;
-  
+
   # Quick multipart creation
   my $quicky = Email::MIME->create(
       header_str => [
@@ -902,7 +934,7 @@ by all means keep reading.
           q[These could be binary too],
       ],
   );
-  
+
   print $email->as_string;
 
 =head1 DESCRIPTION
@@ -1052,7 +1084,7 @@ of additional parts.
   $email->walk_parts(sub {
       my ($part) = @_;
       return if $part->subparts; # multipart
-      
+
       if ( $part->content_type =~ m[text/html]i ) {
           my $body = $part->body;
           $body =~ s/<link [^>]+>//; # simple filter example
@@ -1065,20 +1097,30 @@ each. Accepts a code reference as its only argument. The code reference
 will be passed a single argument, the current MIME part within the
 top-level MIME object. All changes will be applied in place.
 
+=head2 header
+
+B<Achtung!>  Beware this method!  In Email::MIME, it means the same as
+C<header_str>, but on an Email::Simple object, it means C<header_raw>.  Unless
+you always know what kind of object you have, you could get one of two
+significantly different behaviors.
+
+Try to use either C<header_str> or C<header_raw> as appropriate.
+
 =head2 header_str_set
 
   $email->header_str_set($header_name => @value_strings);
 
-This behaves like C<header_set>, but expects Unicode (character) strings as the
-values to set, rather than pre-encoded byte strings.  It will encode them as
-MIME encoded-words if they contain any control or 8-bit characters.
+This behaves like C<header_raw_set>, but expects Unicode (character) strings as
+the values to set, rather than pre-encoded byte strings.  It will encode them
+as MIME encoded-words if they contain any control or 8-bit characters.
 
 =head2 header_str_pairs
 
   my @pairs = $email->header_str_pairs;
 
-This method behaves like C<header_pairs>, returning a list of field name/value
-pairs, but the values have been decoded to character strings, when possible.
+This method behaves like C<header_raw_pairs>, returning a list of field
+name/value pairs, but the values have been decoded to character strings, when
+possible.
 
 =head2 parts
 
@@ -1188,6 +1230,62 @@ Simon Cozens <simon@cpan.org>
 
 =back
 
+=head1 CONTRIBUTORS
+
+=for stopwords Alex Vandiver Arthur Axel 'fREW' Schmidt Brian Cassidy David Steinbrunner Dotan Dimet Geraint Edwards Jesse Luehrs Kurt Anderson Matthew Horsfall (alh) memememomo Shawn Sorichetti Tomohiro Hosaka
+
+=over 4
+
+=item *
+
+Alex Vandiver <alexmv@mit.edu>
+
+=item *
+
+Arthur Axel 'fREW' Schmidt <frioux@gmail.com>
+
+=item *
+
+Brian Cassidy <bricas@cpan.org>
+
+=item *
+
+David Steinbrunner <dsteinbrunner@pobox.com>
+
+=item *
+
+Dotan Dimet <dotan@corky.net>
+
+=item *
+
+Geraint Edwards <gedge-oss@yadn.org>
+
+=item *
+
+Jesse Luehrs <doy@tozt.net>
+
+=item *
+
+Kurt Anderson <kboth@drkurt.com>
+
+=item *
+
+Matthew Horsfall (alh) <wolfsage@gmail.com>
+
+=item *
+
+memememomo <memememomo@gmail.com>
+
+=item *
+
+Shawn Sorichetti <ssoriche@coloredblocks.com>
+
+=item *
+
+Tomohiro Hosaka <bokutin@bokut.in>
+
+=back
+
 =head1 COPYRIGHT AND LICENSE
 
 This software is copyright (c) 2004 by Simon Cozens and Casey West.
@@ -1199,21 +1297,30 @@ the same terms as the Perl 5 programming language system itself.
 
 __END__
 
-
+#pod =method header
+#pod
+#pod B<Achtung!>  Beware this method!  In Email::MIME, it means the same as
+#pod C<header_str>, but on an Email::Simple object, it means C<header_raw>.  Unless
+#pod you always know what kind of object you have, you could get one of two
+#pod significantly different behaviors.
+#pod
+#pod Try to use either C<header_str> or C<header_raw> as appropriate.
+#pod
 #pod =method header_str_set
 #pod
 #pod   $email->header_str_set($header_name => @value_strings);
 #pod
-#pod This behaves like C<header_set>, but expects Unicode (character) strings as the
-#pod values to set, rather than pre-encoded byte strings.  It will encode them as
-#pod MIME encoded-words if they contain any control or 8-bit characters.
+#pod This behaves like C<header_raw_set>, but expects Unicode (character) strings as
+#pod the values to set, rather than pre-encoded byte strings.  It will encode them
+#pod as MIME encoded-words if they contain any control or 8-bit characters.
 #pod
 #pod =method header_str_pairs
 #pod
 #pod   my @pairs = $email->header_str_pairs;
 #pod
-#pod This method behaves like C<header_pairs>, returning a list of field name/value
-#pod pairs, but the values have been decoded to character strings, when possible.
+#pod This method behaves like C<header_raw_pairs>, returning a list of field
+#pod name/value pairs, but the values have been decoded to character strings, when
+#pod possible.
 #pod
 #pod =method parts
 #pod
