@@ -2,7 +2,7 @@ use strict;
 use warnings;
 package Email::MIME::Header;
 # ABSTRACT: the header of a MIME message
-$Email::MIME::Header::VERSION = '1.929';
+$Email::MIME::Header::VERSION = '1.932';
 use parent 'Email::Simple::Header';
 
 use Email::MIME::Encode;
@@ -29,14 +29,14 @@ use Encode 1.9801;
 #pod
 #pod =cut
 
-sub header {
+sub header_str {
   my $self  = shift;
   my $wanta = wantarray;
 
   return unless defined $wanta; # ??
 
-  my @header = $wanta ? $self->SUPER::header(@_)
-                      : scalar $self->SUPER::header(@_);
+  my @header = $wanta ? $self->header_raw(@_)
+                      : scalar $self->header_raw(@_);
 
   local $@;
   foreach my $header (@header) {
@@ -48,10 +48,9 @@ sub header {
   return $wanta ? @header : $header[0];
 }
 
-sub header_raw {
-  Carp::croak "header_raw may not be used to set headers" if @_ > 2;
-  my ($self, $header) = @_;
-  return $self->SUPER::header($header);
+sub header {
+  my ($self, $name) = @_;
+  return $self->header_str($name);
 }
 
 sub header_str_set {
@@ -101,7 +100,7 @@ Email::MIME::Header - the header of a MIME message
 
 =head1 VERSION
 
-version 1.929
+version 1.932
 
 =head1 DESCRIPTION
 
